"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Application = void 0;
const express_1 = __importDefault(require("express"));
const promise_events_1 = require("promise-events");
const context_1 = require("./context");
const octokit_1 = require("./github/octokit");
const logger_1 = require("./logger");
const webhook_event_check_1 = __importDefault(require("./webhook-event-check"));
const wrap_logger_1 = require("./wrap-logger");
// Some events can't get an authenticated client (#382):
function isUnauthenticatedEvent(event) {
    return (!event.payload.installation ||
        (event.name === "installation" && event.payload.action === "deleted"));
}
/**
 * The `app` parameter available to `ApplicationFunction`s
 *
 * @property {logger} log - A logger
 */
class Application {
    constructor(options) {
        const opts = options || {};
        this.events = new promise_events_1.EventEmitter();
        this.log = wrap_logger_1.wrapLogger(logger_1.logger, logger_1.logger);
        this.cache = opts.cache;
        this.router = opts.router || express_1.default.Router(); // you can do this?
        this.githubToken = opts.githubToken;
        this.throttleOptions = opts.throttleOptions;
        this.Octokit = opts.Octokit || octokit_1.ProbotOctokit;
        this.octokit = opts.octokit;
    }
    /**
     * Loads an ApplicationFunction into the current Application
     * @param appFn - Probot application function to load
     */
    load(appFn) {
        if (Array.isArray(appFn)) {
            appFn.forEach((a) => this.load(a));
        }
        else {
            appFn(this);
        }
        return this;
    }
    async receive(event) {
        return Promise.all([
            this.events.emit("*", event),
            this.events.emit(event.name, event),
            this.events.emit(`${event.name}.${event.payload.action}`, event),
        ]);
    }
    /**
     * Get an {@link http://expressjs.com|express} router that can be used to
     * expose HTTP endpoints
     *
     * ```
     * module.exports = app => {
     *   // Get an express router to expose new HTTP endpoints
     *   const route = app.route('/my-app');
     *
     *   // Use any middleware
     *   route.use(require('express').static(__dirname + '/public'));
     *
     *   // Add a new route
     *   route.get('/hello-world', (req, res) => {
     *     res.end('Hello World');
     *   });
     * };
     * ```
     *
     * @param path - the prefix for the routes
     * @returns an [express.Router](http://expressjs.com/en/4x/api.html#router)
     */
    route(path) {
        if (path) {
            const router = express_1.default.Router();
            this.router.use(path, router);
            return router;
        }
        else {
            return this.router;
        }
    }
    on(eventName, callback) {
        if (typeof eventName === "string") {
            void webhook_event_check_1.default(this, eventName);
            return this.events.on(eventName, async (event) => {
                const log = this.log.child({ name: "event", id: event.id });
                try {
                    const github = await this.authenticateEvent(event, log);
                    const context = new context_1.Context(event, github, log);
                    await callback(context);
                }
                catch (err) {
                    // avoid the error.code deprecation message
                    // can be replaced with `log.error({ err, event, ...err })` once @octokit/request-error v3 is used
                    const { name, message, stack, headers, request, status } = err;
                    log.error({
                        err: {
                            name,
                            message,
                            stack,
                        },
                        event,
                        headers,
                        request,
                        status,
                    });
                    throw err;
                }
            });
        }
        else {
            eventName.forEach((e) => this.on(e, callback));
        }
    }
    /**
     * Authenticate and get a GitHub client that can be used to make API calls.
     *
     * You'll probably want to use `context.github` instead.
     *
     * **Note**: `app.auth` is asynchronous, so it needs to be prefixed with a
     * [`await`](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Operators/await)
     * to wait for the magic to happen.
     *
     * ```js
     *  module.exports = (app) => {
     *    app.on('issues.opened', async context => {
     *      const github = await app.auth();
     *    });
     *  };
     * ```
     *
     * @param id - ID of the installation, which can be extracted from
     * `context.payload.installation.id`. If called without this parameter, the
     * client wil authenticate [as the app](https://developer.github.com/apps/building-integrations/setting-up-and-registering-github-apps/about-authentication-options-for-github-apps/#authenticating-as-a-github-app)
     * instead of as a specific installation, which means it can only be used for
     * [app APIs](https://developer.github.com/v3/apps/).
     *
     * @returns An authenticated GitHub API client
     * @private
     */
    async auth(id, log = this.log) {
        if (!id)
            return this.octokit;
        const authOptions = this.githubToken
            ? {}
            : { auth: { installationId: id } };
        const throttleOptions = this.throttleOptions
            ? {
                throttle: {
                    id,
                    ...this.throttleOptions,
                },
            }
            : {};
        const options = {
            log: log.child({ name: "github" }),
            ...authOptions,
            ...throttleOptions,
        };
        return new this.Octokit(options);
    }
    authenticateEvent(event, log) {
        if (this.githubToken) {
            return this.auth();
        }
        if (isUnauthenticatedEvent(event)) {
            log.debug("`context.github` is unauthenticated. See https://probot.github.io/docs/github-api/#unauthenticated-events");
            return this.auth();
        }
        return this.auth(event.payload.installation.id, log);
    }
}
exports.Application = Application;
//# sourceMappingURL=application.js.map